"
COPYRIGHT (c) 2020 LabWare
COPYRIGHT (c) 2022 LabWare

stx:libscm - a new source code management library for Smalltalk/X

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License. 

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"
"{ Package: 'stx:libscm/common' }"

"{ NameSpace: Smalltalk }"

Object subclass:#SCMCopyrightUpdater
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Common-StX-Tools'
!

!SCMCopyrightUpdater class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare
COPYRIGHT (c) 2022 LabWare

stx:libscm - a new source code management library for Smalltalk/X

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License. 

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"
! !

!SCMCopyrightUpdater class methodsFor:'utilities'!

updateClass:aClass
    ^ self new updateClass:aClass
! !

!SCMCopyrightUpdater methodsFor:'private'!

add: missing to: existing inText: lines
    existing do:[:each | 
        (each canBeMergedWith: missing) ifTrue:[ 
            | merged |

            merged := each merge: missing.
            merged prefix: each prefix ? missing prefix.
            lines at: each line put: merged asString.
            ^ self.
        ].
    ].
    "/ Cannot be merged with anything already there,
    "/ add new one.
    existing isEmpty ifTrue:[ 
        "/ No copyright at all, add it to the top...
        lines add: missing asString beforeIndex: 3.
        lines add: ''"empty line" beforeIndex: 4.
    ] ifFalse:[
        missing prefix: existing last prefix ? missing prefix.  
        lines add: missing asString beforeIndex: existing last line + 1
    ].

    "Created: / 27-05-2020 / 12:22:45 / Jan Vrany <jan.vrany@labware.com>"
! !

!SCMCopyrightUpdater methodsFor:'utilities'!

updateClass: aClass
    "Create / update #copyright method in given class"

    | source present current |

    aClass theNonMetaclass programmingLanguage isSmalltalk ifFalse:[ ^self ].

    source := aClass theNonMetaclass copyrightMethodSource asStringCollection.
    present := (SCMCopyrightLine collectCopyrightsFromText: source).
    present notEmpty ifTrue:[ present := present first ].
    current := SCMCopyrightLine new.
    self add: current to: present inText: source.

    aClass theMetaclass compile: source asString classified: 'documentation'.
    (aClass theMetaclass compiledMethodAt:#copyright) setPackage:aClass package.
    "
    SCMCopyrightUpdater new updateClass: Class.  
    SCMCopyrightUpdater new updateClass: UnixOperatingSystem.
    SCMCopyrightUpdater new updateClass: SCMCopyrightLine.  
    SCMCopyrightUpdater new updateClass: Foo.  
    "

    "Created: / 27-05-2020 / 09:19:42 / Jan Vrany <jan.vrany@labware.com>"
    "Modified: / 27-05-2020 / 13:08:06 / Jan Vrany <jan.vrany@labware.com>"
!

updateClass: class usingTemplate: templateText
    "Create / update #copyright method in given class, 
     taking copyright text from given template but *preserve*
     copyright lines from existing copyright!!

    This is useful when re-licensing a package: 

    1. Change the copyright method on package definition class

    2. Execute: 

        pkgDef := <package definition class>.
        template := pkgDef copyrightMethodSource asStringCollection.
        pkgDef classes do:[:class |
            SCMCopyrightUpdater new updateClass: class usingTemplate: template.
        ].   
    "
    
    | oldText oldCopyrights templateCopyrights currentCopyright newText |

    class theNonMetaclass programmingLanguage isSmalltalk ifFalse: [ ^ self ].

    oldText := class theNonMetaclass copyrightMethodSource asStringCollection.
    oldCopyrights := SCMCopyrightLine collectCopyrightsFromText: oldText.
    oldCopyrights notEmpty ifTrue: [
        oldCopyrights := oldCopyrights first
    ].
    templateCopyrights := SCMCopyrightLine collectCopyrightsFromText: templateText.
    templateCopyrights notEmpty ifTrue: [
        templateCopyrights := templateCopyrights first
    ].

    newText := templateText copy.
    "/ Remove template copyrights from new copyright text
    templateCopyrights reverseDo: [ :copyright |
        newText removeAtIndex: copyright line.
    ].
    "/ Copy copyrights from old text to new text (which is based on
    "/ template)
    oldCopyrights do: [:copyright |
        newText add: copyright asString beforeIndex: copyright line.
    ].

    currentCopyright := SCMCopyrightLine new.
    self add: currentCopyright to: oldCopyrights inText: newText.
    class theMetaclass compile: newText asString classified: 'documentation'.
    (class theMetaclass compiledMethodAt: #copyright) setPackage: class package.

    "
     SCMCopyrightUpdater new updateClass: Class.
     SCMCopyrightUpdater new updateClass: UnixOperatingSystem.
     SCMCopyrightUpdater new updateClass: SCMCopyrightLine.
     SCMCopyrightUpdater new updateClass: Foo."

    "Created: / 30-05-2022 / 13:27:06 / Jan Vrany <jan.vrany@labware.com>"
! !

!SCMCopyrightUpdater class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

